@@ -4,8 +4,9 @@ from functools import partial, wraps
 from collections import namedtuple
 
 import torch
+from torch import nn
 import torch.nn.functional as F
-from torch import nn, einsum
+
 from torch.utils.data import Dataset, DataLoader
 from torch.nn.utils.rnn import pad_sequence
 
@@ -21,6 +22,8 @@ from beartype.typing import Callable, Optional, Union, List, Tuple
 from tqdm import tqdm
 from x_clip.tokenizer import tokenizer
 
+pad_sequence = partial(pad_sequence, batch_first = True)
+
 # helpers
 
 def exists(val):
@@ -235,7 +238,6 @@ def sample(
     select_api_start_id_top_k = 10,
 ):
     device = next(model.parameters()).device
-    positions = positions.clone()
     max_seq_len = seq_len + 1
 
     # validate
@@ -255,7 +257,11 @@ def sample(
 
     # sampling positions - different sequences have different cursors
 
-    positions = default(positions, torch.zeros((batch_size,), device = device, dtype = torch.long))
+    if exists(positions):
+        positions = positions.clone()
+    else:
+        positions = torch.zeros((batch_size,), device = device, dtype = torch.long)
+
     assert (positions <= (prime_length + 1)).all() and (positions <= max_seq_len).all(), 'all positions must be less then initial prime length as well as the total sequence length + 1 (plus one for noop if one sequence finished sampling before the other)'
 
     # eval model
@@ -421,6 +427,8 @@ def weight_and_mask(
     return weights.masked_fill(weights == pad_id, 0.)
 
 FilteredResults = namedtuple('FilteredResults', [
+    'num_passed',
+    'num_failed',
     'selected_indices',
     'selected_mask',
     'filtered_tokens',
@@ -511,6 +519,8 @@ def filter_tokens_with_api_response(
     selected_indices = indices[selected_mask]
 
     ret = FilteredResults(
+        selected_mask.sum().item(),
+        (~selected_mask).sum().item(),
         selected_indices,
         selected_mask,
         tokens[selected_mask],
@@ -549,13 +559,29 @@ class PromptDataset(Dataset):
 
 def prompt_collate_fn(data, padding_value = 0):
     prompts, prompt_lengths = zip(*data)
-    prompts = pad_sequence(prompts, batch_first = True, padding_value = padding_value)
+    prompts = pad_sequence(prompts, padding_value = padding_value)
     return prompts, torch.stack(prompt_lengths)
 
 def PromptDataloader(ds: Dataset, *args, padding_value = 0, **kwargs):
     collate_fn = partial(prompt_collate_fn, padding_value = padding_value)
     return DataLoader(ds, *args, collate_fn = collate_fn, **kwargs)
 
+class FinetuneDataset(Dataset):
+    def __init__(
+        self,
+        tokens: torch.Tensor
+    ):
+        self.tokens = tokens
+
+    def __len__(self):
+        return len(self.tokens)
+
+    def __getitem__(self, idx):
+        return self.tokens[idx]
+
+def FinetuneDataloader(ds: Dataset, *args, padding_value = 0, **kwargs):
+    return DataLoader(ds, *args, collate_fn = partial(pad_sequence, padding_value = padding_value), **kwargs)
+
 # classes
 
 @beartype
@@ -578,8 +604,16 @@ class Toolformer(nn.Module):
         model_seq_len = 2048,
         tokenizer_encode: Callable = tokenizer.encode,
         tokenizer_decode: Callable = tokenizer.decode,
+        post_prompt_callback: Callable = identity,
         prompt_input_tag: str = DEFAULT_PROMPT_INPUT_TAG,
-        exclude_filters: dict[str, Callable[[str], bool]] = dict()
+        exclude_filters: dict[str, Callable[[str], bool]] = dict(),
+        finetune = False,
+        finetune_lr = 1e-4,
+        finetune_wd = 1e-2,
+        finetune_betas = (0.9, 0.99),
+        finetune_eps = 1e-8,
+        finetune_epochs = 3,
+        finetune_batch_size = 16
     ):
         super().__init__()
         self.model = model
@@ -589,6 +623,8 @@ class Toolformer(nn.Module):
         self.prompt_batch_size = prompt_batch_size
         self.prompt_input_tag = prompt_input_tag
 
+        self.post_prompt_callback = post_prompt_callback # for easy mocking
+
         self.tokenizer_encode = tokenizer_encode
         self.tokenizer_decode = tokenizer_decode
         self.tokenizer_encode_to_tensor = lambda s: torch.tensor(tokenizer_encode(s)).long()
@@ -624,6 +660,22 @@ class Toolformer(nn.Module):
         self.teach_tool_prompt = teach_tool_prompt
         self.exclude_filters = exclude_filters
 
+        self.should_finetune = finetune
+
+        if not finetune:
+            return
+
+        self.finetune_batch_size = finetune_batch_size
+        self.finetune_epochs = finetune_epochs
+
+        self.optimizer = get_optimizer(
+            model.parameters(),
+            lr = finetune_lr,
+            wd = finetune_wd,
+            betas = finetune_betas,
+            eps = finetune_eps
+        )
+
     def generate_data_with_api_calls(
         self,
         data: List[str],
@@ -661,7 +713,7 @@ class Toolformer(nn.Module):
                 prompted_output = self.tokenizer_decode(sample_output[start_position:])
                 prompted_outputs.append(prompted_output)
 
-        return prompted_outputs
+        return self.post_prompt_callback(prompted_outputs)
 
     def filter_and_keep_only_first_api_call(
         self,
@@ -669,8 +721,15 @@ class Toolformer(nn.Module):
         data_with_api_calls: List[str],
         return_excluded = False
     ):
-        included = []
-        excluded = []
+        included_data = []
+        included_data_with_api_calls = []
+
+        included = (included_data, included_data_with_api_calls)
+
+        excluded_data = []
+        excluded_data_with_api_calls = []
+
+        excluded = (excluded_data, excluded_data_with_api_calls)
 
         api_start_stop_kwargs = dict(api_start = self.api_start_str, api_stop = self.api_stop_str)
 
@@ -680,18 +739,59 @@ class Toolformer(nn.Module):
         for datum, data_with_api_call in zip(data, data_with_api_calls):
             if has_api_calls_(data_with_api_call):
                 data_with_api_call = replace_all_but_first_(data_with_api_call)
-                included.append((datum, data_with_api_call))
-            else:
-                excluded.append((datum, data_with_api_call))
 
-        included = tuple(map(list, zip(*included)))
+                included_data.append(datum)
+                included_data_with_api_calls.append(data_with_api_call)
+            else:
+                excluded_data.append(datum)
+                excluded_data_with_api_calls.append(data_with_api_call)
 
         if not return_excluded:
             return included
 
-        excluded = tuple(map(list, zip(*excluded)))
         return included, excluded
 
+    @torch.no_grad()
+    def sample_model_with_api_calls(
+        self,
+        prime: Union[torch.Tensor, str],
+        occurrence = 1,
+        **kwargs
+    ):
+        self.model.eval()
+
+        prime_is_str = isinstance(prime, str)
+
+        if prime_is_str:
+            prime = self.tokenizer_encode(prime)
+            prime = torch.tensor(prime).long()
+            prime = rearrange(prime, 'n -> 1 n')
+
+        assert prime.shape[0] == 1, 'only one at a time for now'
+
+        invoke_tools_ = partial(invoke_tools, self.registry)
+
+        def call_apis(t: torch.Tensor):
+            t = self.tokenizer_decode(t[0])
+            t = invoke_tools_(t)
+            t =  self.tokenizer_encode_to_tensor(t)
+            return rearrange(t, 'n -> 1 n')
+
+        output = sample_with_api_call(
+            model = self.model,
+            prime = prime,
+            seq_len = self.model_seq_len,
+            call_apis = call_apis,
+            api_end_token_id = self.api_stop_id,
+            occurrence = occurrence,
+            **kwargs
+        )
+
+        if not prime_is_str:
+            return output
+
+        return self.tokenizer_decode(output[0])
+
     def make_api_calls(
         self,
         filtered_data_with_api_calls: List[str]
@@ -717,7 +817,7 @@ class Toolformer(nn.Module):
         data_with_api_responses: List[str]
     ) -> FilteredResults:
 
-        to_token_ids = lambda l: pad_sequence([*map(self.tokenizer_encode_to_tensor, l)], batch_first = True, padding_value = self.pad_id)
+        to_token_ids = lambda l: pad_sequence([*map(self.tokenizer_encode_to_tensor, l)], padding_value = self.pad_id)
 
         tokens, tokens_without_api_response, tokens_with_api_response = map(to_token_ids, (data, data_with_api_calls, data_with_api_responses))
 
@@ -733,15 +833,60 @@ class Toolformer(nn.Module):
 
         return filtered_results
 
+    def finetune(
+        self,
+        filtered_results: Union[FilteredResults, torch.Tensor]
+    ):
+        self.model.train()
+
+        if isinstance(filtered_results, FilteredResults):
+            filtered_results = filtered_results.filtered_tokens_without_api_response
+
+        dataset = FinetuneDataset(tokens = filtered_results)
+        dl = FinetuneDataloader(dataset, batch_size = self.finetune_batch_size, shuffle = True)
+
+        for epoch in tqdm(range(self.finetune_epochs), desc = 'finetune epochs'):
+            for batch in dl:
+                inp, labels = batch[:, :-1], batch[:, 1:]
+
+                logits = self.model(inp)
+                logits = rearrange(logits, 'b n c -> b c n')
+
+                loss = F.cross_entropy(logits, labels, ignore_index = self.pad_id)
+                loss.backward()
+
+                print(f'loss: {loss.item()}')
+                self.optimizer.step()
+                self.optimizer.zero_grad()
+
+        print(f'finished finetuning on {len(dataset)} filtered samples')
+
     def forward(
         self,
-        data: List[str]
+        data: List[str],
+        return_after_generating_api_calls = False,
+        return_after_filtering_api_calls = False,
+        return_after_filtering_by_api_response = False
     ):
         data_with_api_calls = self.generate_data_with_api_calls(data)
-        filtered_data_with_api_calls = self.filter_and_keep_only_first_api_call(data_with_api_calls)
+
+        if return_after_generating_api_calls:
+            return data_with_api_calls
+
+        filtered_data, filtered_data_with_api_calls = self.filter_and_keep_only_first_api_call(data, data_with_api_calls)
+
+        if return_after_filtering_api_calls:
+            return filtered_data, filtered_data_with_api_calls
 
         assert len(filtered_data_with_api_calls) > 0, 'your model failed to follow instructions and make API calls. please try a better model or do some better prompt engineering'
 
         data_with_responses = self.make_api_calls(filtered_data_with_api_calls)
+        filtered_results = self.filter_by_api_responses(filtered_data, filtered_data_with_api_calls, data_with_responses)
 
-        return data_with_responses
+        if return_after_filtering_by_api_response:
+            return filtered_results
+
+        if self.should_finetune:
+            self.finetune(filtered_results)
+
+        return filtered_results
